@@ -1,10 +1,10 @@
-/* $Id: util_ui.h,v 1.37.2.2 2003/09/19 22:42:34 kuhlmann Exp $ */
+/* $Id: util_ui.h,v 1.43 2003/10/09 20:47:31 kuhlmann Exp $ */
 
 #ifndef MICQ_UTIL_UI_H
 #define MICQ_UTIL_UI_H
 
 void UtilUIDisplayMeta (Contact *cont);
-BOOL   Debug (UDWORD level, const char *str, ...) __attribute__ ((format (__printf__, 2, 3)));
+BOOL DebugReal (UDWORD level, const char *str, ...) __attribute__ ((format (__printf__, 2, 3)));
 
 #define DEB_PROTOCOL      0x00000008L
 #define DEB_PACKET        0x00000010L
@@ -13,6 +13,7 @@ BOOL   Debug (UDWORD level, const char *str, ...) __attribute__ ((format (__prin
 #define DEB_EVENT         0x00000080L
 #define DEB_EXTRA         0x00000100L
 #define DEB_CONTACT       0x00000200L
+#define DEB_OPTS          0x00000400L
 #define DEB_PACK5DATA     0x00000800L
 #define DEB_PACK8         0x00001000L
 #define DEB_PACK8DATA     0x00002000L
@@ -24,6 +25,6 @@ BOOL   Debug (UDWORD level, const char *str, ...) __attribute__ ((format (__prin
 #define DEB_IO            0x00400000L
 #define DEB_SSL           0x00800000L
 
-#define AVPFMT COLSERVER "%-15s" COLNONE " %s\n"
+#define AVPFMT "%s%-15s%s %s\n"
 
 #endif /* MICQ_UTIL_UI_H */
