@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: contact.c,v 1.118 2005/02/27 00:12:18 kuhlmann Exp $
+ * $Id: contact.c,v 1.119 2005/04/20 20:14:00 kuhlmann Exp $
  */
 
 #include <ctype.h>
@@ -813,9 +813,9 @@ BOOL ContactMetaLoad (Contact *cont)
             if (!(par = s_parse (&line)))
                 return FALSE;
             enc = ConvEnc (par->txt);
-            if (enc & ENC_FAUTO && (enc ^ prG->enc_loc) & ~ENC_FAUTO)
+            if (enc & ENC_FERR && (enc ^ prG->enc_loc) & ~ENC_FLAGS)
                 return FALSE;
-            enc &= ~ENC_FAUTO;
+            enc &= ~ENC_FLAGS;
         }
         else if (!enc)
             return FALSE;
