@@ -1,4 +1,4 @@
-/* $Id: file_util.c,v 1.199 2005/02/22 22:10:21 kuhlmann Exp $ */
+/* $Id: file_util.c,v 1.204 2005/04/24 11:15:42 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -42,6 +42,7 @@
 #include "oscar_base.h"
 #include "connection.h"
 #include "util_parse.h"
+#include "util_rl.h"
 
 /****/
 
@@ -104,7 +105,9 @@ void Initialize_RC_File ()
     rl_print (i18n (1794, "If you already have an UIN, please enter it. Otherwise, enter 0, and I will request one for you.\n"));
     rl_printf ("%s ", i18n (1618, "UIN:"));
     fflush (stdout);
+    ReadLineTtyUnset ();
     line = UtilIOReadline (stdin);
+    ReadLineTtySet ();
     tmpuin = 0;
     if (line)
         sscanf (line->txt, "%ld", &tmpuin);
@@ -119,9 +122,7 @@ void Initialize_RC_File ()
     {
         rl_printf ("%s ", i18n (1795, "Password:"));
         fflush (stdout);
-        Echo_Off ();
         line = UtilIOReadline (stdin);
-        Echo_On ();
         if (!line)
             continue;
         rl_print ("\n");
@@ -131,10 +132,8 @@ void Initialize_RC_File ()
         rl_print (i18n (1783, "To prevent typos, please enter your password again.\n"));
         rl_printf ("%s ", i18n (1795, "Password:"));
         fflush (stdout);
-        Echo_Off ();
         pwd = strdup (line->txt);
         line = UtilIOReadline (stdin);
-        Echo_On ();
         if (!line)
             continue;
         rl_print ("\n");
@@ -155,6 +154,7 @@ void Initialize_RC_File ()
     rl_print (i18n (1784, "If you are firewalled, you may need to use a SOCKS5 server. If you do, please enter its hostname or IP address. Otherwise, or if unsure, just press return.\n"));
     rl_printf ("%s ", i18n (1094, "SOCKS5 server:"));
     fflush (stdout);
+    ReadLineTtyUnset ();
     line = UtilIOReadline (stdin);
     if (line && line->len)
     {
@@ -201,6 +201,7 @@ void Initialize_RC_File ()
         }
     }
     rl_print ("\n");
+    ReadLineTtySet ();
 
     if (!uin)
     {
@@ -378,18 +379,18 @@ int Read_RC_File (FILE *rcf)
                         rl_printf (i18n (2216, "This mICQ doesn't know the '%s' encoding.\n"), cmd);
                         ERROR;
                     }
-                    if (what & ENC_FAUTO)
+                    if (what & ENC_FERR)
                     {
-                        if (what != (ENC_FAUTO | ENC_UTF8))
+                        if ((what & ~ENC_FLAGS) != ENC_UTF8)
                         {
-                            if ((which == 3 && (what ^ prG->enc_loc) & ~ENC_FAUTO)
-                                || (which == 2 && (what ^ enc) & ~ENC_FAUTO))
+                            if ((which == 3 && (what ^ prG->enc_loc) & ~ENC_FLAGS)
+                                || (which == 2 && (what ^ enc) & ~ENC_FLAGS))
                                 rl_printf ("%s%s%s ", COLERROR, i18n (2251, "Error:"), COLNONE);
                             else
                                 rl_printf ("%s%s%s ", COLERROR, i18n (1619, "Warning:"), COLNONE);
                             rl_printf (i18n (2217, "This mICQ can't convert between '%s' and unicode.\n"), cmd);
                         }
-                        what &= ~ENC_FAUTO;
+                        what &= ~ENC_FLAGS;
                     }
                     if (which == 1)
                     {
@@ -399,11 +400,16 @@ int Read_RC_File (FILE *rcf)
                     }
                     else if (which == 2)
                     {
+                        if (ENC (enc_loc) != what)
+                            prG->locale_broken = 2;
                         prG->enc_loc = what;
-                        prG->locale_broken = 2;
                     }
                     else
+                    {
+                        if (what != ENC_UTF8)
+                            dep = 13;
                         enc = what;
+                    }
                 }
                 else if (!strcasecmp (cmd, "format"))
                 {
@@ -1458,9 +1464,9 @@ void PrefReadStat (FILE *stf)
 int PrefWriteStatusFile (void)
 {
     FILE *stf;
-    char *stfn;
+    char *stfn, *stfo;
     time_t t;
-    int i, k;
+    int i, k, rc;
     Contact *cont;
     Connection *ss;
     ContactGroup *cg;
@@ -1489,7 +1495,10 @@ int PrefWriteStatusFile (void)
             rl_printf (i18n (1872, "failed: %s (%d)\n"), strerror (rc), rc);
     }
     if (!stf)
+    {
+        s_free (stfn);
         return -1;
+    }
 
     t = time (NULL);
     fprintf (stf, "# This file was generated by mICQ " MICQ_VERSION " of %s %s\n", __TIME__, __DATE__);
@@ -1539,17 +1548,26 @@ int PrefWriteStatusFile (void)
     }
 
     if (fclose (stf))
+    {
+        s_free (stfn);
         return -1;
+    }
     
-    return rename (stfn, prG->statfile);
+    stfo = strdup (s_sprintf ("%s.old", prG->statfile));
+    rename (prG->statfile, stfo);
+    s_free (stfo);
+    
+    rc = rename (stfn, prG->statfile);
+    s_free (stfn);
+    return rc;
 }
 
 int PrefWriteConfFile (void)
 {
     FILE *rcf;
-    char *rcfn;
+    char *rcfn, *rcfo;
     time_t t;
-    int k;
+    int k, rc;
     Connection *ss;
 #ifdef ENABLE_TCL
     tcl_pref_p tpref;
@@ -1580,17 +1598,21 @@ int PrefWriteConfFile (void)
             rl_printf (i18n (1872, "failed: %s (%d)\n"), strerror (rc), rc);
     }
     if (!rcf)
+    {
+        s_free (rcfn);
         return -1;
+    }
 
     t = time (NULL);
     fprintf (rcf, "# This file was generated by mICQ " MICQ_VERSION " of %s %s\n", __TIME__, __DATE__);
     fprintf (rcf, "# This file was generated on %s\n", ctime (&t));
 
-    fprintf (rcf, "# Character encodings: auto, iso-8859-1, koi8-u, ...\n");
-    fprintf (rcf, "encoding file UTF-8\n");
+    fprintf (rcf, "encoding file UTF-8 # do not modify\n");
+    fprintf (rcf, "# Local character encodings: auto, ISO-8859-1, KOI8-u, ...\n");
+    fprintf (rcf, "# Setting this explicitly will render mICQ unable to determine character widths.\n");
     fprintf (rcf, "%sencoding local %s%s\n",
-             prG->enc_loc & ENC_FAUTO ? "#" : "", s_quote (ConvEncName (prG->enc_loc)),
-             prG->enc_loc & ENC_FAUTO ? "" : " # please set your locale correctly instead");
+             prG->enc_loc & ENC_FLAGS ? "# " : "", s_quote (ConvEncName (prG->enc_loc)),
+             prG->enc_loc & ENC_FLAGS ? "" : " # please set your locale correctly instead");
     fprintf (rcf, "format 2\n\n");
 
     for (k = 0; (ss = ConnectionNr (k)); k++)
@@ -1710,9 +1732,18 @@ int PrefWriteConfFile (void)
     fprintf (rcf, "\n");
 
     if (fclose (rcf))
+    {
+        s_free (rcfn);
         return -1;
+    }
     
-    return rename (rcfn, prG->rcfile);
+    rcfo = strdup (s_sprintf ("%s.old", prG->rcfile));
+    rename (prG->rcfile, rcfo);
+    s_free (rcfo);
+    
+    rc = rename (rcfn, prG->rcfile);
+    s_free (rcfn);
+    return rc;
 }
 
 int Save_RC ()
