@@ -1,4 +1,4 @@
-/* $Id: icq_response.c,v 1.205 2005/02/26 21:08:06 kuhlmann Exp $ */
+/* $Id: icq_response.c,v 1.207 2005/04/09 11:21:54 kuhlmann Exp $ */
 /* Copyright: This file may be distributed under version 2 of the GPL licence. */
 
 #include "micq.h"
@@ -279,13 +279,13 @@ void HistShow (Contact *cont)
 {
     int i;
     
-    for (i = 0; i < 50; i++)
+    for (i = 0; i < HISTSIZE; i++)
         if (hist[i].conn && (!cont || hist[i].cont == cont))
-            rl_printf ("%s%s %s%*s %c%s %s" COLMSGINDENT "%s\n",
+            rl_printf ("%s%s %s%*s %s%s %s" COLMSGINDENT "%s\n",
                        COLDEBUG, s_time (&hist[i].stamp),
                        hist[i].inout == HIST_IN ? COLINCOMING : COLACK,
                        uiG.nick_len + s_delta (hist[i].cont->nick),
-                       hist[i].cont->nick, hist[i].inout == HIST_IN ? '<' : '>',
+                       hist[i].cont->nick, hist[i].inout == HIST_IN ? "<-" : "->",
                        COLNONE, COLMESSAGE, hist[i].msg);
 }
 
