@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: im_icq8.c,v 1.16 2004/12/22 00:36:11 kuhlmann Exp $
+ * $Id: im_icq8.c,v 1.21 2005/04/24 11:40:56 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -75,6 +75,7 @@ static Contact *IMRosterCheckCont (Connection *serv, RosterEntry *rc)
         tcont->id = 0;
     }
     cont->id = rc->id;
+    cont->tag = rc->tag;
     return cont;
 }
 
@@ -99,7 +100,6 @@ static ContactGroup *IMRosterCheckGroup (Connection *serv, RosterEntry *rg)
         tcg->id = 0;
     }
     cg->id = rg->tag;
-
     return cg;
 }
 
@@ -111,30 +111,59 @@ static void IMRosterDoDelete (Event *event)
     
     if (!roster)
         return;
-    
-    for (re = roster->groups; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
 
-    for (re = roster->ignore; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
+    if (roster->delname)
+    {
+        for (re = roster->groups; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
 
-    for (re = roster->invisible; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
+        for (re = roster->ignore; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
 
-    for (re = roster->visible; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
+        for (re = roster->invisible; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
 
-    for (re = roster->normal; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
+        for (re = roster->visible; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
 
-    for (re = roster->generic; re; re = re->next)
-        if (re->tag == roster->deltag && re->id == roster->delid)
-            SnacCliRosterentrydelete (serv, re);
+        for (re = roster->normal; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->generic; re; re = re->next)
+            if (re->name == roster->delname)
+                SnacCliRosterentrydelete (serv, re);
+    }
+    else
+    {
+        for (re = roster->groups; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->ignore; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->invisible; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->visible; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->normal; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+
+        for (re = roster->generic; re; re = re->next)
+            if (re->tag == roster->deltag && re->id == roster->delid)
+                SnacCliRosterentrydelete (serv, re);
+    }
 
     IMRosterCancel (event);
 }
@@ -155,12 +184,29 @@ static void IMRosterShow (Event *event)
     {
         IMRosterCheckGroup (serv, rg);
         if (prG->verbose)
-            rl_printf ("  %6d", rg->id);
+            rl_printf ("  %6d", rg->tag);
         rl_printf ("  %s%s%s\n",
             COLCONTACT, rg && rg->name ? rg->name : "", COLNONE);
         cnt_groups++;
     }
 
+    rl_printf (i18n (2473, "Normal contacts:\n"));
+    for (rc = roster->normal; rc; rc = rc->next)
+    {
+        cont = IMRosterCheckCont (serv, rc);
+        rg = IMRosterGroup (roster, rc->tag);
+        if (prG->verbose)
+            rl_printf ("  %6d", rc->tag);
+        rl_printf ("  %s%-25s%s %s%15s%s",
+            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
+            COLCONTACT, rc->name, COLNONE);
+        if (prG->verbose)
+            rl_printf ("  %6d", rc->id);
+        rl_printf (" %s%s%s\n",
+            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
+        if (cont)
+            cnt_normal++;
+    }
     rl_printf (i18n (2471, "Ignored contacts:\n"));
     for (rc = roster->ignore; rc; rc = rc->next)
     {
@@ -168,7 +214,7 @@ static void IMRosterShow (Event *event)
         rg = IMRosterGroup (roster, rc->tag);
         if (prG->verbose)
             rl_printf ("  %6d", rc->tag);
-        rl_printf ("  %s%-25s%s %s%-15s%s",
+        rl_printf ("  %s%-25s%s %s%15s%s",
             COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
             COLCONTACT, rc->name, COLNONE);
         if (prG->verbose)
@@ -185,7 +231,7 @@ static void IMRosterShow (Event *event)
         rg = IMRosterGroup (roster, rc->tag);
         if (prG->verbose)
             rl_printf ("  %6d", rc->tag);
-        rl_printf ("  %s%-25s%s %s%-15s%s",
+        rl_printf ("  %s%-25s%s %s%15s%s",
             COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
             COLCONTACT, rc->name, COLNONE);
         if (prG->verbose)
@@ -195,23 +241,6 @@ static void IMRosterShow (Event *event)
         if (cont)
             cnt_hidden++;
     }
-    rl_printf (i18n (2473, "Normal contacts:\n"));
-    for (rc = roster->normal; rc; rc = rc->next)
-    {
-        cont = IMRosterCheckCont (serv, rc);
-        rg = IMRosterGroup (roster, rc->tag);
-        if (prG->verbose)
-            rl_printf ("  %6d", rc->tag);
-        rl_printf ("  %s%-25s%s %s%-15s%s",
-            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
-            COLCONTACT, rc->name, COLNONE);
-        if (prG->verbose)
-            rl_printf ("  %6d", rc->id);
-        rl_printf (" %s%s%s\n",
-            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
-        if (cont)
-            cnt_normal++;
-    }
     rl_printf (i18n (2474, "Intimate contacts:\n"));
     for (rc = roster->visible; rc; rc = rc->next)
     {
@@ -219,7 +248,7 @@ static void IMRosterShow (Event *event)
         rg = IMRosterGroup (roster, rc->tag);
         if (prG->verbose)
             rl_printf ("  %6d", rc->tag);
-        rl_printf ("  %s%-25s%s %s%-15s%s",
+        rl_printf ("  %s%-25s%s %s%15s%s",
             COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
             COLCONTACT, rc->name, COLNONE);
         if (prG->verbose)
@@ -234,14 +263,84 @@ static void IMRosterShow (Event *event)
                cnt_groups, cnt_ignored + cnt_hidden + cnt_intimate + cnt_normal, cnt_ignored, cnt_hidden, cnt_intimate);
 }
 
+static void IMRosterDownCont (Connection *serv, Roster *roster, ContactGroup *cg, RosterEntry *rc, int *pcnt, int *pmod, char force, UDWORD flag)
+{
+    ContactGroup *tcg = NULL;
+    Contact *cont;
+    RosterEntry *rg;
+    char mod = 0;
+    char cnt = 0;
+    
+    rg = IMRosterGroup (roster, rc->tag);
+    cont = IMRosterCheckCont (serv, rc);
+
+    rl_printf ("  %s%-25s%s %s%15s%s %s%s%s\n",
+        COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
+        COLCONTACT, rc->name, COLNONE,
+        COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
+
+    if (!cont)
+        return;
+
+    if (!cont->group)
+    {
+        ContactCreate (serv, cont);
+        ContactAdd (cg, cont);
+        OptSetVal (&cont->copts, CO_IGNORE,   flag == CO_IGNORE);
+        OptSetVal (&cont->copts, CO_HIDEFROM, flag == CO_HIDEFROM);
+        OptSetVal (&cont->copts, CO_INTIMATE, flag == CO_INTIMATE);
+        mod = cnt = 1;
+        (*pcnt)++;
+    }
+    else if (force)
+    {
+        if (ContactPrefVal (cont, CO_LOCAL))
+        {
+            OptSetVal (&cont->copts, CO_LOCAL,    0);
+            mod = 1;
+        }
+        if (ContactPrefVal (cont, CO_IGNORE) != (flag == CO_IGNORE))
+        {
+            OptSetVal (&cont->copts, CO_IGNORE,   flag == CO_IGNORE);
+            mod = 1;
+        }
+        if (ContactPrefVal (cont, CO_HIDEFROM) != (flag == CO_HIDEFROM))
+        {
+            OptSetVal (&cont->copts, CO_HIDEFROM, flag == CO_HIDEFROM);
+            mod = 1;
+        }
+        if (ContactPrefVal (cont, CO_INTIMATE) != (flag == CO_INTIMATE))
+        {
+            OptSetVal (&cont->copts, CO_INTIMATE, flag == CO_INTIMATE);
+            mod = 1;
+        }
+    }
+    if (rg)
+        tcg = IMRosterCheckGroup (serv, rg);
+    if (tcg && !ContactHas (tcg, cont) && (force || cont->group == serv->contacts))
+    {
+        ContactAdd (tcg, cont);
+        cont->group = tcg;
+        mod = 1;
+    }
+    ContactAddAlias (cont, rc->name);
+    if (mod || (rc->nick && strcmp (rc->nick, rc->name) && cont != ContactFind (serv->contacts, 0, cont->uin, rc->nick)))
+    {
+        ContactAddAlias (cont, rc->nick);
+        mod = 1;
+    }
+    if (mod && !cnt)
+        (*pmod)++;
+}
+
 static void IMRosterAdddown (Event *event)
 {
     Roster *roster = event->data;
     RosterEntry *rg, *rc;
     Connection *serv = event->conn;
     ContactGroup *cg;
-    Contact *cont;
     int cnt_groups = 0, cnt_ignored = 0, cnt_hidden = 0, cnt_intimate = 0, cnt_normal = 0;
+    int mod_groups = 0, mod_ignored = 0, mod_hidden = 0, mod_intimate = 0, mod_normal = 0;
     
     if (!roster)
         return;
@@ -251,116 +350,91 @@ static void IMRosterAdddown (Event *event)
         cg = IMRosterCheckGroup (serv, rg);
         if (!cg)
         {
-            rl_printf ("## %d %s\n", rg->id, rg->name);
-            cg = ContactGroupC (serv, rg->id, rg->name);
+            rl_printf ("## %d %s\n", rg->tag, rg->name);
+            cg = ContactGroupC (serv, rg->tag, rg->name);
             OptSetVal (&cg->copts, CO_ISSBL, 1);
             cnt_groups++;
         }
-        if (cg && strcmp (rg->name, cg->name))
+        else if (cg && strcmp (rg->name, cg->name))
         {
             s_repl (&cg->name, rg->name);
             rl_printf ("##> %s\n", cg->name);
-            cnt_groups++;
+            mod_groups++;
         }
     }
     cg = ContactGroupC (NULL, 0, "");
 
-    rl_printf (i18n (2471, "Ignored contacts:\n"));
-    for (rc = roster->ignore; rc; rc = rc->next)
-    {
-        cont = IMRosterCheckCont (serv, rc);
-        rg = IMRosterGroup (roster, rc->tag);
-        if (cont && !cont->group)
-        {
-            ContactCreate (serv, cont);
-            ContactAdd (cg, cont);
-            OptSetVal (&cont->copts, CO_IGNORE, 1);
-            cnt_ignored++;
-        }
-        if (cont)
-        {
-            ContactAddAlias (cont, rc->nick);
-            ContactAddAlias (cont, rc->name);
-        }
-        rl_printf ("  %s%-25s%s %s%-15s%s %s%s%s\n",
-            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
-            COLCONTACT, rc->name, COLNONE,
-            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
-    }
-    rl_printf (i18n (2472, "Hidden-from contacts:\n"));
-    for (rc = roster->invisible; rc; rc = rc->next)
-    {
-        cont = IMRosterCheckCont (serv, rc);
-        rg = IMRosterGroup (roster, rc->tag);
-        if (cont && !cont->group)
-        {
-            ContactCreate (serv, cont);
-            ContactAdd (cg, cont);
-            OptSetVal (&cont->copts, CO_HIDEFROM, 1);
-            OptSetVal (&cont->copts, CO_INTIMATE, 0);
-            cnt_hidden++;
-        }
-        if (cont)
-        {
-            ContactAddAlias (cont, rc->nick);
-            ContactAddAlias (cont, rc->name);
-        }
-        rl_printf ("  %s%-25s%s %s%-15s%s %s%s%s\n",
-            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
-            COLCONTACT, rc->name, COLNONE,
-            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
-    }
     rl_printf (i18n (2473, "Normal contacts:\n"));
     for (rc = roster->normal; rc; rc = rc->next)
-    {
-        cont = IMRosterCheckCont (serv, rc);
-        rg = IMRosterGroup (roster, rc->tag);
-        if (cont && !cont->group)
-        {
-            ContactCreate (serv, cont);
-            ContactAdd (cg, cont);
-            OptSetVal (&cont->copts, CO_HIDEFROM, 0);
-            OptSetVal (&cont->copts, CO_INTIMATE, 0);
-            cnt_normal++;
-        }
-        if (cont)
-        {
-            ContactAddAlias (cont, rc->nick);
-            ContactAddAlias (cont, rc->name);
-        }
-        rl_printf ("  %s%-25s%s %s%-15s%s %s%s%s\n",
-            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
-            COLCONTACT, rc->name, COLNONE,
-            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
-    }
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_normal, &mod_normal, FALSE, 0);
+    rl_printf (i18n (2471, "Ignored contacts:\n"));
+    for (rc = roster->ignore; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_ignored, &mod_ignored, FALSE, CO_IGNORE);
+    rl_printf (i18n (2472, "Hidden-from contacts:\n"));
+    for (rc = roster->invisible; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_hidden, &mod_hidden, FALSE, CO_HIDEFROM);
     rl_printf (i18n (2474, "Intimate contacts:\n"));
     for (rc = roster->visible; rc; rc = rc->next)
-    {
-        cont = IMRosterCheckCont (serv, rc);
-        rg = IMRosterGroup (roster, rc->tag);
-        if (cont && !cont->group)
-        {
-            ContactCreate (serv, cont);
-            ContactAdd (cg, cont);
-            OptSetVal (&cont->copts, CO_HIDEFROM, 0);
-            OptSetVal (&cont->copts, CO_INTIMATE, 1);
-            cnt_intimate++;
-        }
-        if (cont)
-        {
-            ContactAddAlias (cont, rc->nick);
-            ContactAddAlias (cont, rc->name);
-        }
-        rl_printf ("  %s%-25s%s %s%-15s%s %s%s%s\n",
-            COLCONTACT, rg && rg->name ? rg->name : "", COLNONE,
-            COLCONTACT, rc->name, COLNONE,
-            COLCONTACT, rc->nick ? rc->nick : "", COLNONE);
-    }
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_intimate, &mod_intimate, FALSE, CO_INTIMATE);
     if (ContactIndex (cg, 0))
         SnacCliAddcontact (serv, NULL, cg);
     IMRosterCancel (event);
     rl_printf (i18n (2476, "Downloaded %d contact groups, alltogether %d contacts, %d ignored, %d hidden, %d intimate.\n"),
                cnt_groups, cnt_ignored + cnt_hidden + cnt_intimate + cnt_normal, cnt_ignored, cnt_hidden, cnt_intimate);
+    rl_printf (i18n (2578, "Modified %d contact groups, alltogether %d contacts, %d ignored, %d hidden, %d intimate.\n"),
+               mod_groups, mod_ignored + mod_hidden + mod_intimate + mod_normal, mod_ignored, mod_hidden, mod_intimate);
+}
+
+static void IMRosterOverwritedown (Event *event)
+{
+    Roster *roster = event->data;
+    RosterEntry *rg, *rc;
+    Connection *serv = event->conn;
+    ContactGroup *cg;
+    int cnt_groups = 0, cnt_ignored = 0, cnt_hidden = 0, cnt_intimate = 0, cnt_normal = 0;
+    int mod_groups = 0, mod_ignored = 0, mod_hidden = 0, mod_intimate = 0, mod_normal = 0;
+    
+    if (!roster)
+        return;
+
+    for (rg = roster->groups; rg; rg = rg->next)
+    {
+        cg = IMRosterCheckGroup (serv, rg);
+        if (!cg)
+        {
+            rl_printf ("## %d %s\n", rg->tag, rg->name);
+            cg = ContactGroupC (serv, rg->tag, rg->name);
+            OptSetVal (&cg->copts, CO_ISSBL, 1);
+            cnt_groups++;
+        }
+        else if (strcmp (rg->name, cg->name))
+        {
+            s_repl (&cg->name, rg->name);
+            rl_printf ("##> %s\n", cg->name);
+            mod_groups++;
+        }
+    }
+    cg = ContactGroupC (NULL, 0, "");
+
+    rl_printf (i18n (2473, "Normal contacts:\n"));
+    for (rc = roster->normal; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_intimate, &mod_intimate, TRUE, 0);
+    rl_printf (i18n (2471, "Ignored contacts:\n"));
+    for (rc = roster->ignore; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_ignored, &mod_ignored, TRUE, CO_IGNORE);
+    rl_printf (i18n (2472, "Hidden-from contacts:\n"));
+    for (rc = roster->invisible; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_hidden, &mod_hidden, TRUE, CO_HIDEFROM);
+    rl_printf (i18n (2474, "Intimate contacts:\n"));
+    for (rc = roster->visible; rc; rc = rc->next)
+        IMRosterDownCont (serv, roster, cg, rc, &cnt_normal, &mod_normal, TRUE, CO_INTIMATE);
+    if (ContactIndex (cg, 0))
+        SnacCliAddcontact (serv, NULL, cg);
+    IMRosterCancel (event);
+    rl_printf (i18n (2476, "Downloaded %d contact groups, alltogether %d contacts, %d ignored, %d hidden, %d intimate.\n"),
+               cnt_groups, cnt_ignored + cnt_hidden + cnt_intimate + cnt_normal, cnt_ignored, cnt_hidden, cnt_intimate);
+    rl_printf (i18n (2578, "Modified %d contact groups, alltogether %d contacts, %d ignored, %d hidden, %d intimate.\n"),
+               mod_groups, mod_ignored + mod_hidden + mod_intimate + mod_normal, mod_ignored, mod_hidden, mod_intimate);
 }
 
 static void IMRosterAddup (Event *event)
@@ -559,13 +633,14 @@ static void IMRosterDiff (Event *event)
                cnt_ignored, cnt_hidden, cnt_intimate, cnt_more);
 }
 
-UBYTE IMDeleteID (Connection *conn, int tag, int id)
+UBYTE IMDeleteID (Connection *conn, int tag, int id, const char *name)
 {
     Roster *roster;
     
     roster = OscarRosterC ();
     roster->deltag = tag;
     roster->delid = id;
+    roster->delname = name ? strdup (name) : NULL;
     QueueEnqueueData2 (conn, QUEUE_REQUEST_ROSTER, SnacCliCheckroster (conn),
                        900, roster, IMRosterDoDelete, IMRosterCancel);
     return RET_OK;
@@ -587,6 +662,9 @@ UBYTE IMRoster (Connection *serv, int mode)
     switch (mode)
     {
         case IMROSTER_IMPORT:
+            QueueEnqueueData2 (serv, QUEUE_REQUEST_ROSTER, SnacCliCheckroster (serv),
+                               900, NULL, IMRosterOverwritedown, IMRosterCancel);
+            break;
         case IMROSTER_DOWNLOAD:
             QueueEnqueueData2 (serv, QUEUE_REQUEST_ROSTER, SnacCliCheckroster (serv),
                                900, NULL, IMRosterAdddown, IMRosterCancel);
