@@ -1,4 +1,4 @@
-/* $Id: micq.c,v 1.146 2005/02/08 22:47:27 kuhlmann Exp $ */
+/* $Id: micq.c,v 1.150 2005/04/25 16:51:32 kuhlmann Exp $ */
 /* Copyright
  * This file may be distributed under version 2 of the GPL licence.
  */
@@ -290,24 +290,33 @@ static void Init (int argc, char *argv[])
     i18nInit (arg_i);
     PreferencesInit (prG);
     ReadLineInit ();
+    ReadLineAutoExpand ("r", "msg %r");
+    ReadLineAutoExpand ("a", "msg %a");
     
     loaded = arg_h ? 0 : PrefLoad (prG);
     prG->verbose &= ~0x80000000UL;
-    i = i18nOpen (prG->locale);
+    i = i18nOpen (NULL);
 
     if (prG->enc_loc == ENC_AUTO)
-        prG->enc_loc = ENC_FAUTO | ENC_ASCII;
+        prG->enc_loc = ENC_FGUESS | ENC_ASCII;
     
     if (!OptGetVal (&prG->copts, CO_ENCODING, &res))
     {
-        switch (prG->enc_loc & ~ENC_FAUTO)
+        UBYTE enc;
+        switch (ENC (enc_loc))
         {
-            case ENC_EUC:     OptSetVal (&prG->copts, CO_ENCODING, ENC_SJIS); break;
-            case ENC_SJIS:    OptSetVal (&prG->copts, CO_ENCODING, ENC_SJIS); break;
-            case ENC_KOI8:    OptSetVal (&prG->copts, CO_ENCODING, ENC_WIN1251); break;
-            case ENC_WIN1251: OptSetVal (&prG->copts, CO_ENCODING, ENC_WIN1251); break;
-            default:          OptSetVal (&prG->copts, CO_ENCODING, ENC_LATIN1);
+            case ENC_EUC:
+            case ENC_SJIS:
+                enc = ENC_SJIS;
+                break;
+            case ENC_KOI8:
+            case ENC_WIN1251:
+                enc = ENC_WIN1251;
+                break;
+            default:          enc = ENC_LATIN1;
         }
+        OptSetVal (&prG->copts, CO_ENCODING, enc);
+        OptSetStr (&prG->copts, CO_ENCODINGSTR, ConvEncName (enc));
     }
     
     if (arg_c)
