@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oldicq_client.c,v 1.4 2005/02/26 21:08:06 kuhlmann Exp $
+ * $Id: oldicq_client.c,v 1.5 2005/03/10 18:41:33 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -74,7 +74,7 @@ void CmdPktCmdSendMessage (Connection *conn, Contact *cont, const char *text, UD
     pak = PacketCv5 (conn, CMD_SEND_MESSAGE);
     PacketWrite4    (pak, cont->uin);
     PacketWrite2    (pak, type);
-    PacketWriteLNTS (pak, c_out_to (text, cont));
+    PacketWriteLNTS (pak, c_out_to_split (text, cont));
     PacketEnqueuev5 (pak, conn);
 }
 
