@@ -27,7 +27,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: oscar_icbm.c,v 1.18 2005/02/27 22:09:04 kuhlmann Exp $
+ * $Id: oscar_icbm.c,v 1.23 2005/04/09 18:27:26 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -234,7 +234,6 @@ UBYTE SnacCliSendmsg (Connection *serv, Contact *cont, const char *text, UDWORD
     
     switch (format)
     {
-        const char *convtext;
         int remenc;
 
         case 1:
@@ -294,22 +293,11 @@ UBYTE SnacCliSendmsg (Connection *serv, Contact *cont, const char *text, UDWORD
             }
             break;
         case 4:
-            remenc = ContactPrefVal (cont, CO_ENCODING);
-            convtext = ConvTo (text, remenc)->txt;
-            
             PacketWriteTLV     (pak, 5);
             PacketWrite4       (pak, serv->uin);
             PacketWrite1       (pak, type % 256);
             PacketWrite1       (pak, type / 256);
-#if 0
-            PacketWrite2 (pak, strlen (convtext) + strlen (ConvEncName (remenc)) + 2);
-            PacketWriteData (pak, convtext, strlen (convtext));
-            PacketWrite1 (pak, 0);
-            PacketWriteData (pak, ConvEncName (remenc), strlen (ConvEncName (remenc)));
-            PacketWrite1 (pak, 0);
-#else
-            PacketWriteLNTS    (pak, c_out_to (text, cont));
-#endif
+            PacketWriteLNTS    (pak, c_out_to_split (text, cont));
             PacketWriteTLVDone (pak);
             PacketWriteB2 (pak, 6);
             PacketWriteB2 (pak, 0);
@@ -342,7 +330,7 @@ static void SnacCallbackType2Ack (Event *event)
     }
     ASSERT_SERVER (serv);
     
-    IMCliMsg (serv, cont, aevent->opt);
+    IMCliReMsg (serv, cont, aevent->opt);
     aevent->opt = NULL;
     EventD (aevent);
     EventD (event);
@@ -391,7 +379,7 @@ static void SnacCallbackType2 (Event *event)
         return;
     }
     
-    IMCliMsg (serv, cont, event->opt);
+    IMCliReMsg (serv, cont, event->opt);
     event->opt = NULL;
     EventD (event);
 }
@@ -960,12 +948,27 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
     {
         /* Requests for auto-response message */
         do  {
-        case MSGF_GETAUTO | MSG_GET_AWAY:  ack_msg = ContactPrefStr (cont, CO_AUTOAWAY); break;
-        case MSGF_GETAUTO | MSG_GET_OCC:   ack_msg = ContactPrefStr (cont, CO_AUTOOCC);  break;
-        case MSGF_GETAUTO | MSG_GET_NA:    ack_msg = ContactPrefStr (cont, CO_AUTONA);   break;
-        case MSGF_GETAUTO | MSG_GET_DND:   ack_msg = ContactPrefStr (cont, CO_AUTODND);  break;
+                val_t val;
+
+        case MSGF_GETAUTO | MSG_GET_AWAY:
+            ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOAWAY)) && *tauto ? tauto : ContactPrefStr (cont, CO_AUTOAWAY);
+            break;
+        case MSGF_GETAUTO | MSG_GET_OCC:
+            ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTOOCC))  && *tauto ? tauto : ContactPrefStr (cont, CO_AUTOOCC);
+            break;
+        case MSGF_GETAUTO | MSG_GET_NA:
+            ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTONA))   && *tauto ? tauto : ContactPrefStr (cont, CO_AUTONA);
+            break;
+        case MSGF_GETAUTO | MSG_GET_DND:
+            ack_msg = (tauto = ContactPrefStr (cont, CO_TAUTODND))  && *tauto ? tauto : ContactPrefStr (cont, CO_AUTODND);
+            break;
         case MSGF_GETAUTO | MSG_GET_FFC:   ack_msg = ContactPrefStr (cont, CO_AUTOFFC);  break;
-        case MSGF_GETAUTO | MSG_GET_VER:   ack_msg = BuildVersionText;
+        case MSGF_GETAUTO | MSG_GET_VER:
+                if (!OptGetVal (&prG->copts, CO_ENCODING, &val))
+                    val = -1;
+                ack_msg = s_sprintf ("%s\nLocale: %s %s %s %s %d %s", BuildVersionText,
+                                     prG->locale, prG->locale_orig, prG->locale_full,
+                                     ConvEncName (prG->enc_loc), prG->locale_broken, ConvEncName (val));
             } while (0);
 
 #ifdef WIP
@@ -977,16 +980,18 @@ void SrvReceiveAdvanced (Connection *serv, Event *inc_event, Packet *inc_pak, Ev
                 rl_printf ("%s %s%*s%s ", s_now, COLCONTACT, uiG.nick_len + s_delta (cont->nick), cont->nick, COLNONE);
                 if (((ack_flags & TCP_MSGF_INV) && !ContactPrefVal (cont, CO_INTIMATE)) || ContactPrefVal (cont, CO_HIDEFROM))
                 {
-                    rl_printf (i18n (2568, "Ignored request for auto-response from %s%s%s.\n"),
-                               COLCONTACT, cont->nick, COLNONE);
+                    if (ContactPrefVal (cont, CO_SHOWCHANGE))
+                        rl_printf (i18n (2568, "Ignored request for auto-response from %s%s%s.\n"),
+                                   COLCONTACT, cont->nick, COLNONE);
                     ack_event->due = 0;
                     ack_event->callback = NULL;
                     QueueDequeueEvent (ack_event);
                     EventD (ack_event);
                     return;
                 }
-                rl_printf (i18n (1814, "Sent auto-response message to %s%s%s.\n"),
-                           COLCONTACT, cont->nick, COLNONE);
+                if (ContactPrefVal (cont, CO_SHOWCHANGE))
+                    rl_printf (i18n (1814, "Sent auto-response message to %s%s%s.\n"),
+                               COLCONTACT, cont->nick, COLNONE);
             }
 
             accept = TRUE;
