@@ -30,7 +30,7 @@
  * As a special exemption, you may link your binary against OpenSSL,
  * provided your operating system distribution doesn't ship libgnutls.
  *
- * $Id: oscar_roster.c,v 1.17 2005/02/11 23:33:11 kuhlmann Exp $
+ * $Id: oscar_roster.c,v 1.19 2005/04/24 11:38:49 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -79,6 +79,7 @@ void OscarRosterD (Roster *roster)
     OscarRosterEntryD (roster->visible);
     OscarRosterEntryD (roster->invisible);
     OscarRosterEntryD (roster->ignore);
+    s_free (roster->delname);
     free (roster);
 }
 
@@ -216,6 +217,9 @@ JUMP_SNAC_F(SnacSrvReplyroster)
         rref = &roster->generic;
         switch (re->type)
         {
+            case 0: /* normal contact */
+                rref = &roster->normal;
+                break;
             case 1: /* group */
                 if (re->tag || re->id)
                     rref = &roster->groups;
@@ -229,13 +233,10 @@ JUMP_SNAC_F(SnacSrvReplyroster)
             case 14: /* ignore */
                 rref = &roster->ignore;
                 break;
-            case 0: /* normal contact */
-                rref = &roster->normal;
-                break;
+            case 15: /* wierd */
             case 17: /* wierd */
-                SnacCliRosterentrydelete (serv, re);
-                OscarRosterEntryD (re);
-                continue;
+            case 20: /* LastUpdateDate */
+                break;
             case 19: /* ImportTime */
                 j = TLVGet (re->tlv, 212);
                 if (j != (UWORD)-1 && re->tlv[j].str.len == 4)
@@ -576,6 +577,7 @@ JUMP_SNAC_F(SnacSrvUpdateack)
                 if (cont)
                 {
                     cont->id = 0;
+                    cont->tag = 0;
                     rl_printf (i18n (2566, "Contact upload for %s%s%s failed, already on server.\n"),
                                COLCONTACT, s_quote (cont->nick), COLNONE);
                 }
