@@ -28,7 +28,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: preferences.c,v 1.45 2005/02/22 22:10:21 kuhlmann Exp $
+ * $Id: preferences.c,v 1.46 2005/04/24 11:15:42 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -46,6 +46,7 @@
 #include "file_util.h"
 #include "preferences.h"
 #include "util_ui.h"
+#include "conv.h"
 #include "contact.h"
 
 static FILE *PrefOpenRC (Preferences *pref);
@@ -214,6 +215,9 @@ BOOL PrefLoad (Preferences *pref)
             pref->flags &= ~FLAG_AUTOFINGER;
         case 1:
             OptSetVal (&pref->copts, CO_REVEALTIME, 600);
+            if (0)
+        case 2:
+            pref->enc_loc = ENC_AUTO;
         case AUTOUPDATE_CURRENT:
         default:
             pref->autoupdate = AUTOUPDATE_CURRENT;
