@@ -5,7 +5,7 @@
  *  Lots of changes from Rüdiger Kuhlmann. File transfer by Rüdiger Kuhlmann.
  *  This file may be distributed under version 2 of the GPL licence.
  *
- * $Id: tcp.c,v 1.200 2005/02/17 22:52:03 kuhlmann Exp $
+ * $Id: tcp.c,v 1.202 2005/03/31 17:28:15 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -1429,7 +1429,7 @@ BOOL TCPSendFiles (Connection *list, Contact *cont, const char *description, con
     {
         pak = PacketTCPC (peer, TCP_CMD_MESSAGE);
         SrvMsgAdvanced   (pak, peer->our_seq, MSG_FILE, list->parent->status,
-                          cont->status, -1, c_out_to (description, cont));
+                          cont->status, -1, c_out_to_split (description, cont));
         PacketWrite2 (pak, 0);
         PacketWrite2 (pak, 0);
         PacketWriteLNTS (pak, filenames.txt);
@@ -1519,7 +1519,7 @@ static void TCPCallBackResend (Event *event)
         delta = (peer->version > 6 ? 1 : 0);
         if (PacketReadAt2 (pak, 4 + delta) == TCP_CMD_MESSAGE)
         {
-            IMCliMsg (peer->parent->parent, cont, event->opt);
+            IMCliReMsg (peer->parent->parent, cont, event->opt);
             event->opt = NULL;
         }
         else
