@@ -26,7 +26,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_rl.c,v 1.35 2005/02/27 19:26:44 kuhlmann Exp $
+ * $Id: util_rl.c,v 1.40 2005/04/26 21:57:07 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -174,6 +174,14 @@ static UWORD  rl_prompt_len = 0;
 static int    rl_prompt_stat = 0;
 static time_t rl_prompt_time = 0;
 
+typedef struct rl_autoexpandv_s {
+  struct rl_autoexpandv_s *next;
+  char *command;
+  char *replace;
+} rl_autoexpandv_t;
+      
+static rl_autoexpandv_t *rl_ae = NULL;
+
 /*
  * Initialize read line code
  */
@@ -639,13 +647,13 @@ static void rl_analyze_ucs (wint_tt ucs, const char **display, UWORD *columns)
         *columns = 1;
 #if HAVE_WCWIDTH
     else if (!prG->locale_broken && (width = rl_wcwidth (ucs, *display)) != -1)
-        *columns = width;
+        *columns = 256 | width;
 #else
     else if (ucs >= 160) /* no way I'll hard-code double-width or combining marks here */
-        *columns = 1;
+        *columns = 257;
 #endif
     else if (prG->locale_broken && !strcmp (ConvFrom (ConvTo (utf, ENC(enc_loc)), ENC(enc_loc))->txt, utf))
-        *columns = 1;
+        *columns = 257;
     else if (!(ucs & 0xffff0000)) /* more control code, or unknown */
     {
         *display = s_sprintf ("%s\\u%c%c%c%c%s", COLINVCHAR,
@@ -667,6 +675,46 @@ static void rl_analyze_ucs (wint_tt ucs, const char **display, UWORD *columns)
     free (utf);
 }
 
+/*
+ * Determine width and correct display for given (UTF8) string
+ */
+strc_t ReadLineAnalyzeWidth (const char *text, UWORD *width)
+{
+    static str_s str = { NULL, 0, 0 };
+    wchar_tt ucs;
+    UWORD twidth, swidth = 0;
+    const char *dis;
+    int off = 0;
+    str_s in;
+    
+    in.txt = (char *)text;
+    in.len = strlen (text);
+    in.max = 0;
+    s_init (&str, "", 100);
+    
+    for (off = 0; off < in.len; )
+    {
+        ucs = ConvGetUTF8 (&in, &off);
+        rl_analyze_ucs (ucs, &dis, &twidth);
+        swidth += twidth & 0xff;
+        s_cat (&str, twidth & 0x100 ? ConvUTF8 (ucs) : dis);
+    }
+    *width = swidth;
+    return &str;
+}
+
+const char *ReadLinePrintWidth (const char *text, const char *left, const char *right, UWORD *width)
+{
+    UWORD pwidth = *width, twidth;
+    strc_t txt;
+    
+    txt = ReadLineAnalyzeWidth (text, &twidth);
+    if (twidth > pwidth)
+        *width = pwidth = twidth;
+    return s_sprintf ("%s%s%s%*.*s", left, txt->txt, right, pwidth - twidth, pwidth - twidth, "");
+}
+
+
 /*
  * Insert a given unicode codepoint
  */
@@ -676,7 +724,7 @@ static void rl_insert (wint_tt ucs)
     UWORD columns;
 
     rl_analyze_ucs (ucs, &display, &columns);
-    rl_insert_basic (ucs, display, strlen (display), columns);
+    rl_insert_basic (ucs, display, strlen (display), columns & 0xff);
     if (columns && ((rl_ucscol.len > rl_ucspos) || !((rl_prompt_len + rl_colpos) % rl_columns)))
         rl_recheck (FALSE);
 }
@@ -1008,7 +1056,7 @@ static void rl_tab_accept (void)
     {
         wint_tt ucs = ConvGetUTF8 (&str, &off);
         rl_analyze_ucs (ucs, &display, &columns);
-        rl_insert_basic (ucs, display, strlen (display), columns);
+        rl_insert_basic (ucs, display, strlen (display), columns & 0xff);
     }
     rl_tab_state = 0;
     rl_recheck (TRUE);
@@ -1035,7 +1083,7 @@ static void rl_tab_cancel (void)
     {
         wint_tt ucs = ConvGetUTF8 (&str, &off);
         rl_analyze_ucs (ucs, &display, &columns);
-        rl_insert_basic (ucs, display, strlen (display), columns);
+        rl_insert_basic (ucs, display, strlen (display), columns & 0xff);
     }
     rl_tab_state = 0;
     rl_recheck (TRUE);
@@ -1120,7 +1168,7 @@ static void rl_key_tab (void)
         wint_tt ucs = ConvGetUTF8 (&str, &off);
         rl_analyze_ucs (ucs, &display, &columns);
         rl_insert_basic (ucs, s_sprintf ("%s%s%s", rl_colon.txt, display, rl_coloff.txt),
-                         strlen (display) + rl_colon.len + rl_coloff.len, columns);
+                         strlen (display) + rl_colon.len + rl_coloff.len, columns & 0xff);
         rl_tab_len++;
     }
     rl_left (rl_tab_len - rl_tab_common);
@@ -1164,7 +1212,7 @@ static void rl_key_shifttab (void)
         wint_tt ucs = ConvGetUTF8 (&str, &off);
         rl_analyze_ucs (ucs, &display, &columns);
         rl_insert_basic (ucs, s_sprintf ("%s%s%s", rl_colon.txt, display, rl_coloff.txt),
-                         strlen (display) + rl_colon.len + rl_coloff.len, columns);
+                         strlen (display) + rl_colon.len + rl_coloff.len, columns & 0xff);
         rl_tab_len++;
     }
     rl_left (rl_tab_len - rl_tab_common);
@@ -1358,9 +1406,77 @@ static void rl_key_kill (void)
     rl_goto (0);
     rl_linecompress (&rl_temp, 0, -1);
     s_repl (&rl_yank, rl_temp.txt);
+    rl_tab_state = 0;
+    rl_tab_len = 0;
     rl_key_cut ();
 }
 
+/*** read line auto expander ***/
+
+const rl_autoexpand_t *ReadLineListAutoExpand (void)
+{
+    return (const rl_autoexpand_t *) rl_ae;
+}
+
+void ReadLineAutoExpand (const char *command, const char *replace)
+{
+    rl_autoexpandv_t *e;
+    
+    for (e = rl_ae; e; e = e->next)
+    {
+        if (!strcmp (command, e->command))
+        {
+            s_repl (&e->replace, replace);
+            return;
+        }
+    }
+    e = malloc (sizeof (rl_autoexpandv_t));
+    e->next = rl_ae;
+    e->command = strdup (command);
+    e->replace = strdup (replace);
+    rl_ae = e;
+}
+
+static void rl_checkautoexpand (void)
+{
+    rl_autoexpandv_t *e;
+    static str_s rl_exp = { NULL, 0, 0 };
+    char *p, *q;
+    
+    rl_linecompress (&rl_temp, 0, -1);
+        
+    for (e = rl_ae; e; e = e->next)
+        if (!strcmp (rl_temp.txt, e->command))
+            break;
+
+    if (!e)
+        return;
+    
+    s_init (&rl_exp, "", 0);
+    for (p = e->replace; *p && (q = strchr (p, '%')); p = q)
+    {
+        s_catn (&rl_exp, p, q - p);
+        if (q[1] == 'r')
+        {
+            if (uiG.last_rcvd)
+                s_cat (&rl_exp, uiG.last_rcvd->nick ? uiG.last_rcvd->nick : s_sprintf ("%lu", uiG.last_rcvd->uin));
+            q += 2;
+        }
+        else if (q[1] == 'a')
+        {
+            if (uiG.last_sent)
+                s_cat (&rl_exp, uiG.last_sent->nick ? uiG.last_sent->nick : s_sprintf ("%lu", uiG.last_sent->uin));
+            q += 2;
+        }
+        else
+            q++;
+    }
+    s_cat (&rl_exp, p);
+    rl_tab_state = 0;
+    rl_goto (0);
+    rl_lineexpand (rl_exp.txt);
+}
+
 /*** read line input processor ***/
 
 /*
@@ -1459,9 +1575,12 @@ str_t ReadLine (UBYTE newbyte)
                     rl_key_delete_backward_word ();
                     break;
                 case 25:             /* ^Y */
-                    rl_tab_state = 0;
-                    rl_goto (0);
-                    rl_lineexpand (rl_yank);
+                    if (rl_yank)
+                    {
+                        rl_tab_state = 0;
+                        rl_goto (0);
+                        rl_lineexpand (rl_yank);
+                    }
                     break;
                 case 27:             /* ^[ = ESC */
 #ifdef ANSI_TERM
@@ -1471,6 +1590,8 @@ str_t ReadLine (UBYTE newbyte)
                 case 32:             /*   = SPACE */
                     if (rl_tab_state > 0)
                         rl_tab_accept ();
+                    else
+                        rl_checkautoexpand ();
                     rl_insert (' ');
                     break;
                 case 127:            /* DEL */
