@@ -12,12 +12,21 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  * License for more details.
  *
+ * In addition, as a special exception permission is granted to link the
+ * code of this release of mICQ with the OpenSSL project's "OpenSSL"
+ * library, and distribute the linked executables.  You must obey the GNU
+ * General Public License in all respects for all of the code used other
+ * than "OpenSSL".  If you modify this file, you may extend this exception
+ * to your version of the file, but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your version
+ * of this file.
+ *
  * You should have received a copy of the GNU General Public License
  * along with this package; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  *
- * $Id: util_table.c,v 1.26.2.2 2003/05/12 14:38:39 kuhlmann Exp $
+ * $Id: util_table.c,v 1.33 2004/12/05 14:55:31 kuhlmann Exp $
  */
 
 #include "micq.h"
@@ -35,7 +44,7 @@ static const char *TableMonths[] = {   /* *INDENT-OFF* */
 };
 
 static const char *TableLang[] = {
-    _i18n (1100, "None"),
+    "",
     _i18n (1101, "Arabic"),      _i18n (1102, "Bhojpuri"),    _i18n (1103, "Bulgarian"),       _i18n (1104, "Burmese"),
     _i18n (1105, "Cantonese"),   _i18n (1106, "Catalan"),     _i18n (1107, "Chinese"),         _i18n (1108, "Croatian"),
     _i18n (1109, "Czech"),       _i18n (1110, "Danish"),      _i18n (1111, "Dutch"),           _i18n (1112, "English"),
@@ -247,27 +256,26 @@ const char *TableGetMonth (int code)   /* *INDENT-ON* */
 const char *TableGetLang (UBYTE code)
 {
     if (code >= TableLangSize)
-        return i18n (1099, "Unknown language.");
-
+        return NULL;
     return i18n (-1, TableLang[code]);
 }
 
 void TablePrintLang (void)
 {
-    int i;
+    UBYTE i;
     const char *p;
 
     for (i = 1; i < TableLangSize; i++)
     {
         p = i18n (-1, TableLang[i]);
 
-        M_printf ("%2d. %-7s", i, p);
+        rl_printf ("%2d. %-7s", i, p);
         if ((i + 1) & 3)
-            M_print ("\t");
+            rl_print ("\t");
         else
-            M_print ("\n");
+            rl_print ("\n");
     }
-    M_print ("\n");
+    rl_print ("\n");
 }
 
 const char *TableGetCountry (UWORD code)
